<?php
// $Id: translatable.node.inc,v 1.1.2.1 2008/02/20 01:57:25 sun Exp $

/**
 * @file
 * Provides translation services for nodes.
 */

/**
 * Implementation of hook_menu().
 */
function translatable_node_menu($may_cache) {
  $items = array();
  
  if (!$may_cache) {
    if (arg(0) == 'node' && is_numeric(arg(1))) {
      $node = node_load(arg(1));
      $enabled_content_types = variable_get('translatable_content_types', array('page' => 'page', 'story' => 'story'));
      if ($enabled_content_types[$node->type]) {
        $items[] = array(
          'path' => 'node/'. arg(1) .'/translations',
          'title' => t('Translations'),
          'callback' => 'translatable_node_node_page',
          'access' => user_access('administer translatable'),
          'type' => MENU_LOCAL_TASK,
          'weight' => 3,
        );
        if (arg(2) == 'edit' && arg(3) == '' && $node->tnid != $node->nid) {
          drupal_goto('node/'. $node->tnid .'/edit/translation/'. $node->language);
        }
        if (arg(3) == 'translation' && translatable_validate_locale(arg(4))) {
          $items[] = array(
            'path' => 'node/'. arg(1) .'/edit/translation/'. arg(4),
            'title' => t('Edit translation'),
            'callback' => 'translatable_node_edit_translation',
            'callback arguments' => array($node->type .'_node_form', $node, arg(4)),
            'access' => user_access('administer translatable') && node_access('update', $node),
            'weight' => 1,
            'type' => MENU_LOCAL_TASK);
        }
      }
    }
  }
  return $items;
}

/**
 * Menu callback; presents the translation editing form, or redirects to delete confirmation.
 */
function translatable_node_edit_translation($form_id, $node, $lang) {
  if ($_POST['op'] == t('Delete')) {
    // Note: we redirect from node/nid/edit to node/nid/delete to make the tabs disappear.
    if ($_REQUEST['destination']) {
      $destination = drupal_get_destination();
      unset($_REQUEST['destination']);
    }
    drupal_goto('node/'. $node->nid .'/delete', $destination);
  }

  $languages = translatable_available_locales();
  $adminlocale = translatable_set_adminlocale($lang, FALSE);
  drupal_set_title(t('%language translation of !title', array('%language' => $languages[$adminlocale], '!title' => '<em>'. check_plain($node->title) .'</em>')));

  // Check whether we're creating a new translation or editing an existing one.
  $translation = translatable_find('node', "tnid = $node->nid AND language = '$lang'");
  if (!$translation) {
    $node->is_new = TRUE;
  }
  else {
    $node = node_load(key($translation));
  }

  return translatable_get_form($form_id, $node);
}

/**
 * Implementation of hook_nodeapi().
 */
function translatable_nodeapi(&$node, $op, $teaser, $page) {
  $enabled_content_types = variable_get('translatable_content_types', array('page'=>'page', 'story'=>'story'));
  switch ($op) {
    case 'load':
      if ($enabled_content_types[$node->type]) {
        // Add the content's locale
        $properties = translatable_findbyid('node', $node->nid);
        if (!empty($properties['language'])) {
          $properties = array(
            'language' => $properties['language'],
            'tnid' => $properties['tnid'],
            'translatable_node_any' => $properties['any'],
          );
        }
        else {
          $properties = array(
            'language' => translatable_get_defaultcontentlocale(),
            'tnid' => $node->nid,
          );
          if (!empty($node->nid)) {
            $properties['translatable_node_any'] = 0;
          }
        }
        return $properties;
      }
      break;
    
    case 'prepare':
      if (isset($node->language, $node->tnid)) {
        translatable_node_update_fields($node);
      }
      break;
    
    case 'submit':
      // If tnid of the processed node is equal to the node's nid, this is a parent
      // node. We then fetch and process all translations of this node instead.
      if ($node->nid && $node->nid == $node->tnid) {
        $target_nodes = translatable_find('node', "tnid = $node->nid AND nid != $node->nid");
        foreach (array_keys($target_nodes) as $nid) {
          $target_node = node_load($nid);
          translatable_node_update_fields($target_node);
          node_save($target_node);
        }
      }
      break;
      
    case 'insert':
    case 'update':
      if ($enabled_content_types[$node->type]) {
        if (isset($node->language)) {
          if (!isset($node->tnid)) {
            // Set parent node id for new nodes to itself.
            $node->tnid = $node->nid;
          }
          $translatable_node = array(
            'nid' => $node->nid,
            'tnid' => $node->tnid,
            'language' => $node->language,
            'any' => (int)$node->translatable_node_any,
          );
          translatable_save('node', $translatable_node);
        }
      }
      break;
    
    case 'delete':
      if ($enabled_content_types[$node->type]) {
        translatable_delete('node', $node->nid);
      }
      break;
  }
}

function translatable_node_form_alter($form_id, &$form) {
  if (!isset($form['type']) || !isset($form['type']['#value'])) {
    return;
  }
  $type = $form['type']['#value'];
  if ($form_id != $type .'_node_form') {
    // Exit if this is not a node form.
    return;
  }

  // If this content type is enabled for translation, render a translation
  // language selection list and add a checkbox to display a content/translation
  // in all languages.
  $enabled_content_types = variable_get('translatable_content_types', drupal_map_assoc(array('page', 'story')));
  if ($enabled_content_types[$type]) {
    // If we are on node/#/edit (a source node), our translation callback has
    // not been executed, so we need to ensure that the proper translation
    // language is set.
    if (arg(3) != 'translation') {
      translatable_set_adminlocale($form['#node']->language, FALSE);
    }
    
    // Enable translation language block.
    translatable_adminlocale_formselect($form_id, $form);

    // If configured, enable any language by default for new nodes.
    if (!isset($form['#node']->translatable_node_any)) {
      $form['#node']->translatable_node_any = variable_get('translatable_default_content_any', FALSE) ? 1 : 0;
    }
    $form['translatable_node_any'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display this content in any language'),
      '#default_value' => $form['#node']->translatable_node_any,
      '#weight' => 0,
    );
    // Disable any language property if it is already set for another language.
    $anynode = translatable_find('node', 'tnid = '. (int)$form['#node']->tnid .' AND nid != '. (int)$form['#node']->nid .' AND any = 1', FALSE);
    if ($anynode) {
      $form['translatable_node_any']['#disabled'] = TRUE;
      $form['translatable_node_any']['#description'] = t('Another translation is currently displayed in any language. <a href="!any">Click here to switch to this translation.</a>', array('!any' => url('node/'. $anynode['nid'] .'/edit')));
    }

    // Remove certain fields when creating or editing a new node and we're not
    // in the default locale.  Otherwise, contents of these fields would be
    // stored in the default locale while the node is stored in the current
    // admin locale.
    // @todo Taxonomy free tagging
    if (translatable_get_default_locale() != translatable_get_adminlocale()) {
      $form['menu']['#access'] = FALSE;
    }
  }
}

/**
 * Callback invoked form translatable_get_form().
 *
 * Prepopulates a node form with source content values and disables
 * untranslatable form fields.
 */
function translatable_node_alter_form($form_id, &$form) {
  if (!isset($form['type']) || !isset($form['type']['#value'])) {
    return;
  }
  $type = $form['type']['#value'];
  if ($form_id != $type .'_node_form') {
    // Exit if this is not a node form.
    return;
  }

  // Creating a new translation requires more work.
  if ($form['#node']->is_new) {
    global $user;

    // Load the source node.
    $source_node = node_load(arg(1));

    /*
    // Let 3rd party modules prepare certain form fields that need extra treatment.
    $_form = array();
    foreach (module_implements('translatable_prepare_node') as $module) {
      $function = $module .'_translatable_prepare_node';
      $result = $function($source_node);
      if (isset($result) && is_array($result)) {
        $_form = array_merge($_form, $result);
      }
    }
    */

    // Reset some values to defaults.
    $source_node->nid = NULL;
    $form['nid']['#value'] = NULL;
    $source_node->vid = NULL;
    $form['vid']['#value'] = NULL;
    $source_node->uid = $user->uid;
    $form['uid']['#value'] = $user->uid;
    $source_node->created = '';
    $form['created']['#value'] = '';
    if (isset($source_node->menu)) {
      $source_node->menu = NULL;
    }
    if (isset($source_node->path)) {
      $source_node->path = NULL;
    }
    // Replace the stored node in the form
    $form['#node'] = $form['#parameters'][1] = $source_node;

    // Switch language
    translatable_set_adminlocale(arg(4), FALSE);
    
    // Generate a clean translation form and merge values from the source node form.
    $new_node = array(
      'uid' => $user->uid,
      'name' => $user->name,
      'type' => $type,
      'tnid' => $source_node->tnid,
      'language' => arg(4),
    );
    $new_form = translatable_retrieve_form($form_id, $new_node);
    $new_form['#is_new'] = TRUE;
    // Give modules a chance to assign #translatable after all form alterations
    // are done.  This is needed, because modules like CCK fieldgroup perform
    // excessive alterations.
    foreach (module_implements('translatable_form_alter') as $module) {
      $function = $module .'_translatable_form_alter';
      $function($form_id, $new_form);
    }
    translatable_filter_form($new_form, $form);
    $form = $new_form;

    $form['tnid'] = array(
      '#type' => 'value',
      '#value' => $form['#node']->tnid,
    );
    $form['#node']->translatable_node_any = 0;
  }
  else {
    // Editing an existing translation: load the translation's parent node.
    $parent_node = node_load($form['#node']->tnid);
    // Generate the parent editing form and use it to filter current values.
    $parent_form = translatable_retrieve_form($form_id, $parent_node);
    // Give modules a chance to assign #translatable after all form alterations
    // are done.  This is needed, because modules like CCK fieldgroup perform
    // excessive alterations.
    foreach (module_implements('translatable_form_alter') as $module) {
      $function = $module .'_translatable_form_alter';
      $function($form_id, $form);
    }
    translatable_filter_form($form, $parent_form);
  }
  
  // Set parent node id.
  if (!isset($form['tnid'])) {
    $form['tnid'] = array(
      '#type' => 'value',
      '#value' => $form['#node']->tnid,
    );
  }
  
  // Determine and set target language for new translations.
  if (!isset($form['#node']->language)) {
    $form['#node']->language = translatable_get_adminlocale();
  }
  $form['language'] = array(
    '#type' => 'hidden',
    '#default_value' => $form['#node']->language,
  );
  
  // Ensure that untranslatable fields are not altered.
  $form['#submit'] = array('translatable_filter_form_submit' => array($form)) + (array)$form['#submit'];
}

/**
 * Updates fields in translations of a node.
 *
 * If a node translation is saved, all untranslatable values have to be
 * copied from the source node into the translation node.
 *
 * If a source node is saved, all translations of this node have to be
 * updated to synchronize hidden changes.
 *
 * @param &$node
 *    A node object to process.
 *
 * @todo Update to translatable_get_translatable_fields().
 */
function translatable_node_update_fields(&$node) {
  if ($node->nid == $node->tnid) {
    return;
  }
  // Retrieve a list of all node fields that have to be updated.
  $update_fields = module_invoke_all('translatable_fields', 'node_form');

  // Load field values from parent node.
  $source_node = node_load($node->tnid);

  // Update fields in translated node.
  foreach ($source_node as $field => $value) {
    if (isset($update_fields[$field])) {
      $node->$field = $value;
    }
  }
}

/**
 * Implementation of hook_link_alter().
 */
function translatable_link_alter(&$node, &$links) {
  if ($node->language != translatable_get_locale()) {
    foreach ($links as $id => $link) {
      if (isset($links[$id]['query'])) {
        $links[$id]['query'] = $links[$id]['query'] .'&locale='. $node->language;
      }
      else {
        $links[$id]['query'] = 'locale='. $node->language;
      }
    }
  }
}

/**
 * Intercepts all the calls for the page.
 */
function translatable_node_node_page() {
  $args = func_get_args();
  $op = array_shift($args);
  
  return translatable_page_translations($op, $args);
}

/**
 * Get the id of a node that is a translation of a given node in a given language.
 * 
 * @todo Make this function generic and move it to translatable.database.inc.
 *
 * @param $nid
 *   The parent node id.
 * @param $_locale
 *   The locale to search for.
 *
 * @return
 *   The corresponding translated node's nid or FALSE if no translation exists.
 */
function translatable_node_get_localizednid($nid, $locale) {
  $translatable_node = translatable_findbyid('node', $nid);
  if ($locale != $translatable_node['language']) {
    if ($tnid = $translatable_node['tnid']) {
      $translatable_node = translatable_find('node', "tnid = $tnid AND language = '$locale'", FALSE);
      return $translatable_node['nid'];
    }
  }
  return FALSE;
}

function translatable_node_get_contentlocale($alias) {
  $path = drupal_get_normal_path($alias);
  $args = explode('/', $path);
  $language = '';
  if ($args[0] == 'node' && is_numeric($args[1])) {
    $translatable_node = translatable_findbyid('node', $args[1]);
    $language = $translatable_node['language'];
  }
  return $language;
}

function translatable_node_existscontentlocale($alias, $language) {
  $path = drupal_get_normal_path($alias);
  $args = explode('/', $path);
  $nid = '';
  if ($args[0] == 'node' && is_numeric($args[1])) {
    $translatable_node = translatable_findbyid('node', $args[1]);
    if ($tnid = $translatable_node['tnid']) {
      $translatable_node = translatable_find('node', "tnid = $tnid AND language = '$language'", FALSE);
      $nid = $translatable_node['nid'];
    }
  }
  if ($nid) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * Temporary hard-coded #translatable field properties.
 */
function translatable_translatable_form_alter($form_id, &$form) {
  // Synchronize general node fields and properties.
  $form['title']['#translatable'] = TRUE;
  if (!empty($form['body_filter'])) {
    $form['body_filter']['#translatable'] = TRUE;
  }
  $form['options']['#translatable'] = TRUE;
  
  // Synchronize menu options for nodes; we default to Localizer (not i18n)
  // translations currently, so menu items are translated elsewhere.  Since
  // Drupal assumes the default locale as default language for all menu items,
  // menu items can only be added or edited in the default language.
  if (translatable_get_default_locale() != translatable_get_adminlocale()) {
    $form['menu']['#translatable'] = FALSE;
  }

  // Synchronize comment options from source.
  if (module_exists('comment')) {
    $form['comment_settings']['#translatable'] = TRUE;
  }
  
  // Disable taxonomy fields; we default to Localizer (not i18n) translations
  // currently, so taxonomies are translated elsewhere.
  if (module_exists('taxonomy')) {
    foreach (element_children($form['taxonomy']) as $key => $vocab) {
      $form['taxonomy'][$vocab]['#translatable'] = FALSE;
    }
  }
}

